@@ -168,8 +168,8 @@ _OP = _define_opcodes(_OPCODES)
 
 # The regular expression flags.
 _REGEX_FLAGS = {"a": ASCII, "i": IGNORECASE, "L": LOCALE, "m": MULTILINE,
-  "r": REVERSE, "s": DOTALL, "u": UNICODE, "w": WORD, "x": VERBOSE, "z":
-  ZEROWIDTH}
+  "r": REVERSE, "s": DOTALL, "u": UNICODE, "w": WORD, "x": VERBOSE,
+  "z": ZEROWIDTH}
 
 # Unicode properties.
 _PROPERTIES = """\
@@ -1916,7 +1916,7 @@ class _Range(_RegexBase):
         flags = int(self.positive) + _ZEROWIDTH_OP * int(self.zerowidth)
         return [(self._opcode[reverse], flags, self.min_value, self.max_value)]
     def dump(self, indent=0, reverse=False):
-        print "%sRANGE %s %s %s" % (_INDENT * indent, self._pos_text[self.positive], self.min_value, self.max_value)
+        print "%s%s %s %s %s" % (_INDENT * indent, self._op_name[reverse], self._pos_text[self.positive], self.min_value, self.max_value)
     def get_range(self):
         return (self.min_value, self.max_value)
     def is_case_sensitive(self, info):
@@ -2074,8 +2074,15 @@ class _Set(_RegexBase):
     _opcode = {False: _OP.SET, True: _OP.SET_REV}
     _op_name = {False: "SET", True: "SET_REV"}
     _pos_text = {False: "NON-MATCH", True: "MATCH"}
-    _big_bitset_opcode = {False: _OP.BIG_BITSET, True: _OP.BIG_BITSET_REV}
-    _small_bitset_opcode = {False: _OP.SMALL_BITSET, True: _OP.SMALL_BITSET_REV}
+    _big_bitset_opcode = {
+      (False, False): _OP.BIG_BITSET, (False, True): _OP.BIG_BITSET_REV,
+      (True, False): _OP.BIG_BITSET_IGN, (True, True): _OP.BIG_BITSET_IGN_REV
+    }
+    _small_bitset_opcode = {
+      (False, False): _OP.SMALL_BITSET, (False, True): _OP.SMALL_BITSET_REV,
+      (True, False): _OP.SMALL_BITSET_IGN, (True, True): _OP.SMALL_BITSET_IGN_REV
+    }
+    _ignore = False
     def __init__(self, members, positive=True, zerowidth=False):
         self.members, self.positive, self.zerowidth = members, bool(positive), zerowidth
         self._key = self.__class__, tuple(self.members), self.positive, self.zerowidth
@@ -2152,19 +2159,19 @@ class _Set(_RegexBase):
                 else:
                     return self._as_range(r[0], r[1]).compile(reverse)
             else:
-                return self._make_bitset(ranges, reverse)
+                return self._make_bitset(ranges, self._ignore, reverse)
 
         # If there's only a single other then compile to just that.
         if not ranges and len(others) == 1:
             o = others[0]
-            o.positive = self.positive
+            o.positive = o.positive == self.positive
             o.zerowidth = self.zerowidth
             return o.compile(reverse)
 
         # Compile a compound set.
         flags = int(self.positive) + _ZEROWIDTH_OP * int(self.zerowidth)
         code = [(self._opcode[reverse], flags)]
-        code.extend(self._make_bitset(ranges, reverse))
+        code.extend(self._make_bitset(ranges, False, False))
         for o in others:
             code.extend(o.compile())
         code.append((-self._opcode[reverse], ))
@@ -2183,7 +2190,7 @@ class _Set(_RegexBase):
     CODE_MASK = (1 << BITS_PER_CODE) - 1
     CODES_PER_SUBSET = 256 // BITS_PER_CODE
     SUBSET_MASK = (1 << 256) - 1
-    def _make_bitset(self, ranges, reverse):
+    def _make_bitset(self, ranges, ignore, reverse):
         code = []
         # values for big bitset are: max_char indexes... subsets...
         # values for small bitset are: top_bits bitset
@@ -2220,13 +2227,14 @@ class _Set(_RegexBase):
             for subset, ind in sorted(subset_index.items(), key=lambda pair: pair[1]):
                 data.extend(self._bitset_to_codes(subset))
             flags = int(self.positive) + _ZEROWIDTH_OP * int(self.zerowidth)
-            code.append((self._big_bitset_opcode[reverse], flags, max_char) + tuple(data))
+            code.append((self._big_bitset_opcode[(ignore, reverse)], flags, max_char) + tuple(data))
         else:
             # Build a small bitset.
             flags = int(self.positive) + _ZEROWIDTH_OP * int(self.zerowidth)
             for top_bits, bitset in bitset_dict.items():
                 if bitset:
-                    code.append((self._small_bitset_opcode[reverse], flags, top_bits) + tuple(self._bitset_to_codes(bitset)))
+                    code.append((self._small_bitset_opcode[(ignore, reverse)],
+                      flags, top_bits) + tuple(self._bitset_to_codes(bitset)))
         return code
     def _bitset_to_codes(self, bitset):
         codes = []
@@ -2242,8 +2250,7 @@ class _Set(_RegexBase):
 class _SetIgn(_Set):
     _opcode = {False: _OP.SET_IGN, True: _OP.SET_IGN_REV}
     _op_name = {False: "SET_IGN", True: "SET_IGN_REV"}
-    _big_bitset_opcode = {False: _OP.BIG_BITSET_IGN, True: _OP.BIG_BITSET_IGN_REV}
-    _small_bitset_opcode = {False: _OP.SMALL_BITSET_IGN, True: _OP.SMALL_BITSET_IGN_REV}
+    _ignore = True
     def optimise(self, info):
         # Case-sensitive matches are faster, so convert to a case-sensitive
         # instance if all the members are case-insensitive.
