@@ -1280,6 +1280,40 @@ class Test:
         self.expect(lambda: regex.match(r"(?|(?<a>a)(?<b>b)|(c)(?<a>d))(e)",
           "cde"), self.DUPLICATE_GROUP)
 
+    def test_set(self):
+        self.expect(lambda: regex.match(r"[a]", "a").span(), "(0, 1)")
+        self.expect(lambda: regex.match(r"(?i)[a]", "A").span(), "(0, 1)")
+        self.expect(lambda: regex.match(r"[a-b]", r"a").span(), "(0, 1)")
+        self.expect(lambda: regex.match(r"(?i)[a-b]", r"A").span(), "(0, 1)")
+
+        self.expect(lambda: regex.findall(ur"[\p{Alpha}]", u"a0"), 
+          repr([u"a"]))
+        self.expect(lambda: regex.findall(ur"(?i)[\p{Alpha}]", u"A0"), 
+          repr([u"A"]))
+
+        self.expect(lambda: regex.findall(ur"[a\p{Alpha}]", u"ab0"), 
+          repr([u"a", u"b"]))
+        self.expect(lambda: regex.findall(ur"[a\P{Alpha}]", u"ab0"), 
+          repr([u"a", u"0"]))
+        self.expect(lambda: regex.findall(ur"(?i)[a\p{Alpha}]", u"ab0"), 
+          repr([u"a", u"b"]))
+        self.expect(lambda: regex.findall(ur"(?i)[a\P{Alpha}]", u"ab0"), 
+          repr([u"a", u"0"]))
+
+        self.expect(lambda: regex.findall(ur"[a-b\p{Alpha}]", u"abC0"), 
+          repr([u"a", u"b", u"C"]))
+        self.expect(lambda: regex.findall(ur"(?i)[a-b\p{Alpha}]", u"AbC0"), 
+          repr([u"A", u"b", u"C"]))
+
+        self.expect(lambda: regex.findall(ur"[\p{Alpha}]", u"a0"), 
+          repr([u"a"]))
+        self.expect(lambda: regex.findall(ur"[\P{Alpha}]", u"a0"), 
+          repr([u"0"]))
+        self.expect(lambda: regex.findall(ur"[^\p{Alpha}]", u"a0"), 
+          repr([u"0"]))
+        self.expect(lambda: regex.findall(ur"[^\P{Alpha}]", u"a0"), 
+          repr([u"a"]))
+
     def test_various(self):
         tests = [
             # Test ?P< and ?P= extensions.
