@@ -1301,6 +1301,40 @@ class Test:
         self.expect(lambda: regex.match(r"(?|(?<a>a)(?<b>b)|(c)(?<a>d))(e)",
           "cde"), self.DUPLICATE_GROUP)
 
+    def test_set(self):
+        self.expect(lambda: regex.match(r"[a]", "a").span(), "(0, 1)")
+        self.expect(lambda: regex.match(r"(?i)[a]", "A").span(), "(0, 1)")
+        self.expect(lambda: regex.match(r"[a-b]", r"a").span(), "(0, 1)")
+        self.expect(lambda: regex.match(r"(?i)[a-b]", r"A").span(), "(0, 1)")
+
+        self.expect(lambda: regex.findall(r"[\p{Alpha}]", "a0"), 
+          ascii(["a"]))
+        self.expect(lambda: regex.findall(r"(?i)[\p{Alpha}]", "A0"), 
+          ascii(["A"]))
+
+        self.expect(lambda: regex.findall(r"[a\p{Alpha}]", "ab0"), 
+          ascii(["a", "b"]))
+        self.expect(lambda: regex.findall(r"[a\P{Alpha}]", "ab0"), 
+          ascii(["a", "0"]))
+        self.expect(lambda: regex.findall(r"(?i)[a\p{Alpha}]", "ab0"), 
+          ascii(["a", "b"]))
+        self.expect(lambda: regex.findall(r"(?i)[a\P{Alpha}]", "ab0"), 
+          ascii(["a", "0"]))
+
+        self.expect(lambda: regex.findall(r"[a-b\p{Alpha}]", "abC0"), 
+          ascii(["a", "b", "C"]))
+        self.expect(lambda: regex.findall(r"(?i)[a-b\p{Alpha}]", "AbC0"), 
+          ascii(["A", "b", "C"]))
+
+        self.expect(lambda: regex.findall(r"[\p{Alpha}]", "a0"), 
+          ascii(["a"]))
+        self.expect(lambda: regex.findall(r"[\P{Alpha}]", "a0"), 
+          ascii(["0"]))
+        self.expect(lambda: regex.findall(r"[^\p{Alpha}]", "a0"), 
+          ascii(["0"]))
+        self.expect(lambda: regex.findall(r"[^\P{Alpha}]", "a0"), 
+          ascii(["a"]))
+
     def test_various(self):
         tests = [
             # Test ?P< and ?P= extensions.
